from django import forms
from django.utils.safestring import mark_safe
from django.core.validators import RegexValidator, URLValidator
from jabberfr.validators import validate_jid_userpart, validate_jid
from jabberfr.utils import pick_captcha, validate_captcha

USERNAME_HELP = """
<ul>
  <li>Ceci est le <em>nom d'utilisateur</em> que vous utiliserez pour vous identifier sur votre compte.</li>
  <li>Le nom d'utilisateur est insensible à la casse, <em>votrenom</em> est idententique à <em>VotreNom</em>.</li>
  <li>Un nom d'utilisateur ne peut contenir <span style="text-decoration: underline;">aucun</span> de ces caractères : @ : ' " &lt; &gt; &amp;</li>
  <li>Certains serveurs limitent la taille du nom d'utilisateur à 16 caractères.</li>
</ul>
"""

SERVER_HELP = """
<ul>
  <li>Vous pouvez utiliser certains services de serveurs autres que celui sur lequel vous êtes inscrit, comme par exemple les salons de discussions.</li>
</ul>
"""


PASSWORD_HELP = """
<ul>
  <li>Ce mot de passe protège votre compte. Ne le divulguez à personne.
      Personne d'autre que vous n'a besoin de votre mot de passe.</li>
  <li>Vous pouvez toujours utiliser votre client Jabber pour changer votre mot de passe.</li>
  <li>Vous pouvez prendre soin d'indiquer votre adresse e-mail dans les informations personnelles de votre compte afin de pouvoir vous faire envoyer votre mot de passe à l'aide du <a href="https://im.apinc.org/outils/password.php">formulaire</a>, autrement il vous sera impossible de récupérer le mot de passe en cas d'oubli.</li>
</ul>
"""


class HebergForm(forms.Form):
    """Form for hosted domains"""
    domain = forms.CharField(
        max_length=100,
        required=True,
        label='Votre nom de domaine',
        validators=[
            RegexValidator(
                regex=URLValidator.host_re,
                message='Ce nom de domaine n’est pas valide'
            )
        ],
    )
    email = forms.EmailField(
        required=True,
        label='Votre adresse email',
    )
    type = forms.ChoiceField(
        choices=[
            ('users', 'Comptes utilisateurs'),
            ('muc', 'Salons'),
        ],
        label='Type d’hébergement',
        required=True,
    )
    message = forms.CharField(
        widget=forms.Textarea,
        max_length=3000,
        label='Un petit mot ?'
    )
    cname = forms.BooleanField(
        required=True,
        label='J’ai effectué la redirection CNAME',
    )
    cgu = forms.BooleanField(
        required=True,
        label=(
            'J’accepte les CGU'
        )
    )


def help_hack(txt: str) -> str:
    """Required to preserve html structure with our help text"""
    return '</span></p>' + txt + '<p><span>'


class InscriptionForm(forms.Form):
    username = forms.CharField(
        help_text=mark_safe(help_hack(USERNAME_HELP)),
        label='Nom d’utilisateur',
        validators=[validate_jid_userpart],
        required=True,
    )
    server = forms.ChoiceField(
        choices=[
            ('jabber.fr', 'jabber.fr'),
            ('im.apinc.org', 'im.apinc.org'),
        ],
        label='Serveur Jabber',
        required=True,
        help_text=mark_safe(help_hack(SERVER_HELP)),
        validators=[
            RegexValidator(
                regex=URLValidator.host_re,
                message='Ce nom de domaine n’est pas valide'
            )
        ],
    )
    password = forms.CharField(
        widget=forms.PasswordInput,
        label='Mot de passe',
        required=True,
    )
    confirm_password = forms.CharField(
        widget=forms.PasswordInput,
        label='Vérification du mot de passe',
        required=True,
        help_text=mark_safe(help_hack(PASSWORD_HELP)),
    )

    def __init__(self, *args, **kwargs):
        choices = kwargs.pop('custom_choices', [])
        super().__init__(*args, **kwargs)
        if choices:
            self.fields['server'].widget.choices = choices

    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data['password'] != cleaned_data['confirm_password']:
            self.add_error(
                'confirm_password', 'Les mots de passe ne correspondent pas.'
            )


class AdhesionForm(forms.Form):
    surname = forms.CharField(
        required=True,
        label="Nom de famille",
    )
    name = forms.CharField(
        required=True,
        label='Prénom',
    )
    email = forms.EmailField(
        required=True,
        label='Adresse email',
    )
    jid = forms.CharField(
        required=True,
        label='Identififant XMPP (JID)',
        validators=[validate_jid],
    )
    captcha_id = forms.IntegerField(
        required=True,
        widget=forms.HiddenInput,
    )
    captcha = forms.CharField(
        required=True,
        label='Captcha',
        help_text='',
    )

    def __init__(self, *args, **kwargs):
        captcha_id, captcha_question = kwargs.pop('captcha', (0, ''))
        super().__init__(*args, **kwargs)
        if captcha_id and captcha_question:
            self.reset_captcha(captcha_id, captcha_question)

    def reset_captcha(self, captcha_id: int, question: str):
        self.initial['captcha_id'] = captcha_id
        self.fields['captcha_id'].initial = captcha_id
        self.initial['captcha'] = ''
        self.fields['captcha'].initial = ''
        self.fields['captcha'].help_text = mark_safe(
            f'\n<br/>Répondez à cette question : {question}'
        )

    def clean(self):
        cleaned_data = super().clean()
        if not validate_captcha(cleaned_data):
            self.add_error('captcha', 'Êtes-vous un robot ?')
            self.reset_captcha(*pick_captcha())


def reset_captcha_form(form: forms.Form):
    form_data = {key: value[0] for key, value in dict(form.data).items()}
    form_data.pop('captcha', None)
    form_data.pop('captcha_id', None)
    cap_id, question = pick_captcha()
    form_data['captcha_id'] = str(cap_id)
    new_form = AdhesionForm(
        data=form_data, initial=form.initial
    )
    new_form.fields['captcha'].help_text = mark_safe(
        f'\n<br/>Répondez à cette question : {question}'
    )
    new_form._errors = form._errors
    return new_form
