import logging
import xml.etree.ElementTree as ET
from asyncio import gather, get_event_loop
from random import randint
from uuid import uuid4

from aiodns import DNSResolver
from aiohttp import ClientSession, ClientTimeout
from aiohttp.web import (
    HTTPConflict,
    HTTPServiceUnavailable,
)
from django.conf import settings
from slixmpp import JID, InvalidJID

log = logging.getLogger(__name__)

BASE_MUC_URL = 'http://[::1]:5280/muc_list/?'

DOMAINS_URL = 'https://jabberfr.org/domains.xml'

REGISTER_URL = 'http://[::1]:5280/register_account/'
VERIFY_URL = REGISTER_URL + 'verify'


async def get_chatrooms(*, limit: int = 25, order: str = 'users') -> str:
    params = {
        'limit': str(limit),
        'order': order,
        'class': 'lastposts',
    }
    params_str = ';'.join(f'{key}={value}' for key, value in params.items())
    url = BASE_MUC_URL + params_str

    tmout = ClientTimeout(total=10)
    async with ClientSession(raise_for_status=True, timeout=tmout) as session:
        async with session.get(url) as resp:
            return await resp.text()


async def get_custom_domains() -> set[str]:
    results = set()
    tmout = ClientTimeout(total=10)
    async with ClientSession(raise_for_status=True, timeout=tmout) as session:
        try:
            async with session.get(DOMAINS_URL) as resp:
                text = await resp.text()
                root = ET.fromstring(text)
                for child in root:
                    results.add(child.attrib['jid'])
        except BaseException:
            log.error('Unable to parse custom domains', exc_info=True)
    return results


def check_srv_contents(srv_client, srv_server) -> bool:
    if not isinstance(srv_client, Exception):  # No SRV
        for srv in srv_client:
            if srv.host != 'jabberfr.org' or srv.port != 5222:
                return True
    if not isinstance(srv_server, Exception):  # No SRV
        for srv in srv_server:
            if srv.host != 'jabberfr.org' or srv.port != 5269:
                return True
    return False


def format_srv_errors(srv_client, srv_server) -> str:
    msg = (
        '<p>Vous avez des enregistrements SRV qui ne pointent pas '
        'vers JabberFR:</p>\n'
    )
    records = []
    if not isinstance(srv_client, Exception):
        for srv in srv_client:
            records.append(srv)
    if not isinstance(srv_server, Exception):
        for srv in srv_server:
            records.append(srv)
    text = '<pre>'

    for rec in records:
        text += (
            f'{rec.host}. {rec.type} {rec.ttl} {rec.priority} '
            f'{rec.weight} {rec.port}\n'
        )
    text += '</pre>'

    return msg + text


async def check_dns_config(data: dict) -> list[str]:
    errors = []
    resolver = DNSResolver(loop=get_event_loop())
    cname, srv_client, srv_server = await gather(
        resolver.query(data['domain'], 'CNAME'),
        resolver.query(f'_xmpp-server._tcp.{data["domain"]}', 'SRV'),
        resolver.query(f'_xmpp-client._tcp.{data["domain"]}', 'SRV'),
        return_exceptions=True,
    )
    if check_srv_contents(srv_client, srv_server):
        errors.append(format_srv_errors(srv_client, srv_server))
    if isinstance(cname, BaseException):
        errors.append(
            f'<p>Vous n’avez pas d’enregistrement <code>CNAME</code> pour ce'
            f' domaine, ajoutez-en un qui pointe vers JabberFR :\n'
            f'<pre><code>{data["domain"]} IN CNAME jabberfr.org</code></pre>'
        )
    elif cname.cname != 'jabberfr.org':
        errors.append(
            f'<p>Votre enregistrement <code>CNAME</code> ne pointe pas '
            f'vers <code>jabberfr.org</code> mais vers '
            f'<code>{cname.cname}</code>,  vous devriez changer ça dans '
            f'votre interface de gestion DNS.</p>'
        )
    return errors


def pick_captcha() -> tuple[str, str]:
    choice = randint(0, len(settings.CAPTCHA_QUESTIONS) - 1)
    question = settings.CAPTCHA_QUESTIONS[choice][0]
    return str(choice), question


def validate_captcha(data: dict) -> bool:
    try:
        captcha_id = data['captcha_id']
        if captcha_id < 0 or captcha_id >= len(settings.CAPTCHA_QUESTIONS):
            raise ValueError
    except ValueError:
        return False
    question_tup = settings.CAPTCHA_QUESTIONS[captcha_id]
    if question_tup is None:
        return False
    if question_tup[1] != data['captcha']:
        return False
    return True


class RegistrationError(Exception):
    def __init__(self, text):
        super().__init__()
        self.text = text


async def register_account(username: str, host: str, password: str, ip: str):
    """Register an account using prosody’s mod_register_json"""
    try:
        jid = JID(f'{username}@{host}')
    except InvalidJID:
        return
    headers = {'Host': jid.server}
    # we don’t care about the email feature
    email = uuid4().hex + '@example.com'
    payload = {
        'username': jid.user,
        'password': password,
        'ip': ip,
        'auth_token': settings.REGISTRATION_TOKEN,
        'mail': email,
    }

    tmout = ClientTimeout(total=10)
    async with ClientSession(raise_for_status=True, timeout=tmout) as session:
        try:
            async with session.post(REGISTER_URL, json=payload, headers=headers) as resp:
                uid = await resp.text()
            async with session.post(VERIFY_URL, data={'uid': uid}, headers=headers):
                pass
        except HTTPConflict:
            raise RegistrationError(
                'Ce nom d’utilisateur n’est pas disponible.'
            )
        except HTTPServiceUnavailable:
            raise RegistrationError(
                'Vous avez fait trop de demandes, veuillez revenir plus tard.'
            )
        except BaseException:
            raise RegistrationError(
                'Erreur inconnue. (serveur indisponible ?)'
            )


def get_client_ip(request) -> str:
    """Get a client IP"""
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip
