import logging
from aiohttp import ClientError
from django.shortcuts import render
from django.core.mail import send_mail
from jabberfr.forms import (
    HebergForm,
    InscriptionForm,
    AdhesionForm,
    reset_captcha_form,
)
from jabberfr.utils import (
    get_chatrooms,
    get_custom_domains,
    check_dns_config,
    pick_captcha,
    validate_captcha,
    register_account,
    RegistrationError,
    get_client_ip,
)

log = logging.getLogger(__name__)


TABLE_END = """
    <td colspan="5"><a href="//chat.jabberfr.org" style="display: table-cell; font-weight: bold; line-height: 2em">Voir plus…</a></td>
  </tr>
</table>
"""


async def get_root_index(request):
    try:
        rooms = await get_chatrooms(limit=6)
        rooms = rooms[:-9] + TABLE_END
    except ClientError:
        rooms = '<p>Impossible de récupérer la liste des salons !</p>'
    context = {
        'rooms': rooms,
    }
    return render(request, 'index.html', context)


async def get_chat_index(request):
    try:
        rooms = await get_chatrooms()
    except ClientError:
        rooms = '<p>Impossible de récupérer la liste des salons !</p>'
    context = {
        'title': 'Salons de discussion de JabberFR',
        'rooms': rooms,
    }

    return render(request, 'chat/index.html', context=context)


async def hebergement(request):
    context = {
        'title': 'Hébergement chez JabberFR',
    }
    if request.method == 'POST':
        form = HebergForm(request.POST)
        context['form'] = form
        if form.is_valid():
            errors = await check_dns_config(form)
            if not errors:
                body = (
                    f'{form.email} demande que {form.domain} soit hébergé par '
                    f'JabberFR, type {form.type}, et a dit :\n{form.message}'
                )
                try:
                    send_mail(
                        subject='Demande d’hébergement',
                        message=body,
                        from_email='hebergement@jabberfr.org',
                        recipient_list=['association@jabberfr.org'],
                    )
                except BaseException:
                    log.error(
                        'ATTENTION: Le message n’a pas été envoyé :\n%s',
                        body,
                        exc_info=True,
                    )
                    form.add_error(
                        '__all__',
                        'Impossible d’envoyer la demande, merci de nous '
                        'contacter directement.'
                    )
                else:
                    return render(request, 'hebergement_success.html', context)
            context['errors'] = errors
    else:
        form = HebergForm()
        context['form'] = form
    return render(request, 'hebergement.html', context=context)


async def inscription(request, domain=None):
    if domain and domain not in await get_custom_domains():
        domain = None
    custom = domain
    context = {'custom': custom}
    if not domain:
        choices = [
            ('jabber.fr', 'jabber.fr'),
            ('im.apinc.org', 'im.apinc.org'),
        ]
    else:
        choices = [(domain, domain)]
    if request.method == 'POST':
        form = InscriptionForm(request.POST, custom_choices=choices)
        context['form'] = form
        if form.is_valid():
            try:
                data = form.cleaned_data
                await register_account(
                    data['username'],
                    data['server'],
                    data['password'],
                    get_client_ip(request),
                )
            except RegistrationError as exc:
                form.add_error('__all__', exc.text)
            else:
                context['jid'] = data['username'] + '@' + data['server']
                return render(
                    request,
                    'inscription_success.html',
                    context=context
                )
    else:
        form = InscriptionForm(custom_choices=choices)
        context['form'] = form
    return render(request, 'inscription.html', context=context)


async def adhesion(request):
    context = {}
    if request.method == 'POST':
        form = AdhesionForm(request.POST)
        context['form'] = form
        if form.is_valid():
            data = form.cleaned_data
            body = (
                f'Nouvelle adhésion à l’association :\n\n'
                f'Nom : {data["surname"]}\nPrénom : {data["name"]}\n'
                f'Email : {data["email"]}\nJID : {data["jid"]}\n'
                f'----'
            )
            try:
                send_mail(
                    subject='Nouvelle adhésion à l’association',
                    message=body,
                    from_email='no-reply@jabberfr.org',
                    recipient_list=['tresorier@jabberfr.org'],
                )
            except BaseException:
                log.error(
                    'ATTENTION: Erreur d’envoi de mail:\n%s',
                    body,
                    exc_info=True,
                )
                form.add_error(
                    '__all__',
                    'Impossible d’envoyer la demande, merci de nous '
                    'contacter directement.'
                )
                context['form'] = reset_captcha_form(form)
            else:
                return render(
                    request,
                    'adhesion_success.html',
                    context=context
                )
        else:
            context['form'] = reset_captcha_form(form)
    else:
        form = AdhesionForm(captcha=pick_captcha())
        context['form'] = form
    return render(request, 'adhesion.html', context=context)
